from setuptools import setup
from os import path as os_path

this_directory = os_path.abspath(os_path.dirname(__file__))


def read_file(filename):
    with open(os_path.join(this_directory, filename), encoding='utf-8') as f:
        long_description = f.read()
    return long_description


setup(
  name='redis_distributed_lock',
  version="1.2",
  description="Redis distributed lock for python",
  long_description=read_file('README.md'),
  long_description_content_type="text/markdown",
  license="MIT",
  keywords='redis distributed lock',
  author='halukshan',
  author_email='halukshan@gmail.com',
  url='https://github.com/halukshan/redis-distributed-lock',
  classifiers=['Development Status :: 4 - Beta',
               'Intended Audience :: Developers',
               'Natural Language :: English',
               'Operating System :: POSIX :: Linux',
               'Programming Language :: Python :: 3.7'
  ],
  packages=['redis_distributed_lock'],
)


import time
import uuid


class RDLock:
    def __init__(self, redisconn, sleeptime=100, prefix='lock_'):
        self.redisconn = redisconn
        self.sleeptime = sleeptime / 1000
        self.prefix = prefix
        self.value = str(uuid.uuid1())
        self.script = """
                    if redis.call('get', KEYS[1]) == ARGV[1] then
                        return redis.call('del', KEYS[1])
                    else
                        return 0
                    end
                """

    def acquire(self, key, expire=5000, timeout: int = None):
        """ Acquire a redis distribution lock from redis connection with block mode
        :param
            expire: int
                key expire time (millisecond)
            timeout: int
                before timeout, keep trying (second)
        """
        if timeout: assert isinstance(timeout, int)
        while True:
            if self.redisconn.set(self.prefix + key, self.value, nx=True, px=expire):
                return True
            if timeout:
                if timeout <= 0: return False
                timeout -= self.sleeptime
            time.sleep(self.sleeptime)

    def acquire_no_block(self, key, expire=5000):
        """ Acquire a redis distribution lock from redis connection with No-block mode"""
        if self.redisconn.set(self.prefix + key, self.value, nx=True, px=expire):
            return True
        else:
            return False

    def release(self, key):
        script = self.redisconn.register_script(self.script)
        script(keys=[self.prefix + key], args=[self.value])
